#!/usr/bin/python
# Fixmultcoords - Program for fixing co-ords of multiple conformers
# To be run after CNS 1.0 alternate.inp
# Version 1.0
# Copyright Stephen Graham 2001
# History
#	17/7/01		Initial version (1.0)

import os
import string
import sys

if len(sys.argv) != 3:
	print 'Script for adding correct co-ordinates to the output of alternate.inp.'
	print 'Currently only works for 2 conformations where the conformations are called AC1 and AC2'
	print 'Usage:	fixmultcoords <PDB file (main conf)> <PDB file (alt conf)>'
	sys.exit(1)

mainConf = open( sys.argv[1], "rw" )
altConf = open( sys.argv[2], "rw" )

# Read all of the main conf into a dictionary
lines = mainConf.readlines()
hashTable = {}
i = 0
for ln in lines:
	if ln[0:4] == "ATOM":
		key = ln[13:20] + ln[22:26] + ln[72:75]
		hashTable[key] = i
	i = i+1

# Go through the alt conf and update co-ords line by line
for altLn in altConf.readlines():
	if altLn[0:4] == "ATOM":
		key = altLn[13:20] + altLn[22:26] + "AC2"
		if not hashTable.has_key( key ):
			print "Error: Atom "+key+" not present in main conformer"
			print "Bailing out"
			altConf.close()
			mainConf.close()
			sys.exit(1)
		lineNo = hashTable[key]
		mainLn = lines[lineNo]

		# Now we have the line, modify the co-ordinates
		newCoOrds = altLn[31:54]
		newLn = mainLn[:31] + newCoOrds + mainLn[54:]
		lines[lineNo] = newLn

# Having updated the co-ords, just print out the PDB file and we are done
out = sys.stdout
out.write( "REMARK Alternate conformation co-ords updated by fixmulcoords 1.0\n" )
out.writelines( lines )

# Clean everything up
altConf.close()
mainConf.close()
out.close()
